@@ -47,8 +47,7 @@ mergeInto(LibraryManager.library, {
     callStackNameToId: {},
     callStackIdToName: {},
     callStackId: 0,
-    afterUnwind: null,
-    asyncFinalizers: [], // functions to run when *all* asynchronicity is done
+    asyncPromiseHandlers: null, // { resolve, reject } pair for when *all* asynchronicity is done
     sleepCallbacks: [], // functions to call every time we sleep
 
     getCallStackId: function(funcName) {
@@ -75,16 +74,34 @@ mergeInto(LibraryManager.library, {
               try {
                 return original.apply(null, arguments);
               } finally {
-                // Only functions in the list of known relevant imports are allowed to change the state.
-                // Note that invoke_* functions are allowed to change the state if we do not ignore
-                // indirect calls.
+                // Only asyncify-declared imports are allowed to change the
+                // state.
+                var isAsyncifyImport = ASYNCIFY_IMPORTS.indexOf(x) >= 0 ||
+                                       x.startsWith('__asyncjs__');
+                // Changing the state from normal to disabled is allowed (in any
+                // function) as that is what shutdown does (and we don't have an
+                // explicit list of shutdown imports).
+                var changedToDisabled =
+                      originalAsyncifyState === Asyncify.State.Normal &&
+                      Asyncify.state        === Asyncify.State.Disabled;
+                // invoke_* functions are allowed to change the state if we do
+                // not ignore indirect calls.
+                var ignoredInvoke = x.startsWith('invoke_') &&
+                                    {{{ !ASYNCIFY_IGNORE_INDIRECT }}};
                 if (Asyncify.state !== originalAsyncifyState &&
-                    ASYNCIFY_IMPORTS.indexOf(x) < 0 &&
-                    !(x.startsWith('invoke_') && {{{ !ASYNCIFY_IGNORE_INDIRECT }}})) {
-                  throw 'import ' + x + ' was not in ASYNCIFY_IMPORTS, but changed the state';
+                    !isAsyncifyImport &&
+                    !changedToDisabled &&
+                    !ignoredInvoke) {
+                  throw new Error('import ' + x + ' was not in ASYNCIFY_IMPORTS, but changed the state');
                 }
               }
             }
+#if MAIN_MODULE
+            // The dynamic library loader needs to be able to read .sig
+            // properties, so that it knows function signatures when it adds
+            // them to the table.
+            imports[x].sig = original.sig;
+#endif
           }
         })(x);
       }
@@ -141,20 +158,19 @@ mergeInto(LibraryManager.library, {
         if (typeof Fibers !== 'undefined') {
           Fibers.trampoline();
         }
-        if (Asyncify.afterUnwind) {
-          Asyncify.afterUnwind();
-          Asyncify.afterUnwind = null;
-        }
       }
     },
 
     whenDone: function() {
 #if ASSERTIONS
       assert(Asyncify.currData, 'Tried to wait for an async operation when none is in progress.');
-      assert(!Asyncify.asyncFinalizers.length, 'Cannot have multiple async operations in flight at once');
+      assert(!Asyncify.asyncPromiseHandlers, 'Cannot have multiple async operations in flight at once');
 #endif
-      return new Promise(function(resolve) {
-        Asyncify.asyncFinalizers.push(resolve);
+      return new Promise(function(resolve, reject) {
+        Asyncify.asyncPromiseHandlers = {
+          resolve: resolve,
+          reject: reject
+        };
       });
     },
 
@@ -247,7 +263,15 @@ mergeInto(LibraryManager.library, {
           if (typeof Browser !== 'undefined' && Browser.mainLoop.func) {
             Browser.mainLoop.resume();
           }
-          var asyncWasmReturnValue = Asyncify.doRewind(Asyncify.currData);
+          var asyncWasmReturnValue, isError = false;
+          try {
+            asyncWasmReturnValue = Asyncify.doRewind(Asyncify.currData);
+          } catch (err) {
+            asyncWasmReturnValue = err;
+            isError = true;
+          }
+          // Track whether the return value was handled by any promise handlers.
+          var handled = false;
           if (!Asyncify.currData) {
             // All asynchronous execution has finished.
             // `asyncWasmReturnValue` now contains the final
@@ -261,11 +285,18 @@ mergeInto(LibraryManager.library, {
             // contains the return value of the exported WASM function
             // that may have called C functions that
             // call `Asyncify.handleSleep()`.
-            var asyncFinalizers = Asyncify.asyncFinalizers;
-            Asyncify.asyncFinalizers = [];
-            asyncFinalizers.forEach(function(func) {
-              func(asyncWasmReturnValue);
-            });
+            var asyncPromiseHandlers = Asyncify.asyncPromiseHandlers;
+            if (asyncPromiseHandlers) {
+              Asyncify.asyncPromiseHandlers = null;
+              (isError ? asyncPromiseHandlers.reject : asyncPromiseHandlers.resolve)(asyncWasmReturnValue);
+              handled = true;
+            }
+          }
+          if (isError && !handled) {
+            // If there was an error and it was not handled by now, we have no choice but to
+            // rethrow that error into the global scope where it can be caught only by
+            // `onerror` or `onunhandledpromiserejection`.
+            throw asyncWasmReturnValue;
           }
         });
         reachedAfterCallback = true;
@@ -362,16 +393,19 @@ mergeInto(LibraryManager.library, {
     });
   },
 
+  emscripten_scan_registers__deps: ['$safeSetTimeout'],
   emscripten_scan_registers: function(func) {
     Asyncify.handleSleep(function(wakeUp) {
-      // We must first unwind, so things are spilled to the stack. We
-      // can resume right after unwinding, no need for a timeout.
-      Asyncify.afterUnwind = function() {
+      // We must first unwind, so things are spilled to the stack. Then while
+      // we are pausing we do the actual scan. After that we can resume. Note
+      // how using a timeout here avoids unbounded call stack growth, which
+      // could happen if we tried to scan the stack immediately after unwinding.
+      safeSetTimeout(function() {
         var stackBegin = Asyncify.currData + {{{ C_STRUCTS.asyncify_data_s.__size__ }}};
         var stackEnd = HEAP32[Asyncify.currData >> 2];
         {{{ makeDynCall('vii', 'func') }}}(stackBegin, stackEnd);
         wakeUp();
-      };
+      }, 0);
     });
   },
 
